//
// log.cc         : Log Filter
// author         : Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: log.cc,v 1.1 2001/09/07 23:10:27 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include "log.hh"

LogFilter *app;

char *msg_types[] = {"INTEREST", "POSITIVE REINFORCEMENT",
		     "NEGATIVE REINFORCEMENT", "DATA",
		     "FLAGGED DATA", "CONTROL", "REDIRECT"};

void MyFilterReceive::recv(Message *msg, handle h)
{
  app->recv(msg, h);
}

void LogFilter::recv(Message *msg, handle h)
{
  if (h != filterHandle){
    fprintf(stderr, "Error: recv received message for handle %d when subscribing to handle %d !\n", h, filterHandle);
    return;
  }

  ProcessMessage(msg);

  ((DiffusionRouting *)dr)->sendMessageToNext(msg, h);
}

void LogFilter::ProcessMessage(Message *msg)
{
  fprintf(stderr, "Received a");

  if (msg->new_message)
    fprintf(stderr, " new ");
  else
    fprintf(stderr, "n old ");

  if (msg->last_hop != LOCALHOST_ADDR)
    fprintf(stderr, "%s message from node %d\n", msg_types[msg->msg_type],
	    msg->last_hop);
  else
    fprintf(stderr, "%s message from agent %d\n", msg_types[msg->msg_type],
	    msg->source_port);
}

handle LogFilter::setupFilter()
{
  NRAttrVec attrs;
  handle h;

  // This is a dummy attribute for filtering
  // It matches everything
  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));

  h = ((DiffusionRouting *)dr)->addFilter(&attrs, LOG_FILTER_PRIORITY, fcb);

  ClearAttrs(&attrs);
  return h;
}

void LogFilter::run()
{
  // Doesn't do anything
  while (1){
    sleep(10);
  }
}

void LogFilter::usage()
{
  fprintf(stderr, "Usage: logging [-p port] [-h]\n\n");
  fprintf(stderr, "\t-p - Uses port 'port' to talk to diffusion\n");
  fprintf(stderr, "\t-h - Prints this information\n");
  fprintf(stderr, "\n");
  exit(0);
}

LogFilter::LogFilter(int argc, char **argv)
{
  int opt;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  // Parse command line options
  opterr = 0;

  while (1){
    opt = getopt(argc, argv, "hp:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	fprintf(stderr, "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 'h':

      usage();

      break;

    case '?':

      fprintf(stderr, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage();

      break;

    case ':':

      fprintf(stderr, "Parameter missing !\n");
      usage();

      break;

    }

    if (opt == -1)
      break;
  }

  // Create Diffusion Routing class
  dr = NR::createNR(diffusion_port);
  fcb = new MyFilterReceive;

  // Set up the filter
  filterHandle = setupFilter();
  fprintf(stderr, "Log filter subscribed to *, received handle %d\n", filterHandle);

  fprintf(stderr, "Log filter initialized !\n");
}

int main(int argc, char **argv)
{
  // Initialize and run the Log Filter
  app = new LogFilter(argc, argv);
  app->run();

  return 0;
}
