//
// test.cc : demonstrates type-safe attribute handling
// authors : John Heidemann and Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: test.cc,v 1.4 2001/05/23 19:12:41 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include "test.hh"

/*
 * demonstrate declaring user attributes
 */
NRSimpleAttributeFactory<double> HighPrecisionLatAttr(3001, NRAttribute::FLOAT64_TYPE);
NRSimpleAttributeFactory<char*> GroceriesAttr(3002, NRAttribute::STRING_TYPE);
NRSimpleAttributeFactory<char*> NonGroceriesAttr(3003, NRAttribute::STRING_TYPE);
NRSimpleAttributeFactory<char *> TaskNameAttr(NRAttribute::TASK_NAME_KEY, NRAttribute::STRING_TYPE);
NRSimpleAttributeFactory<void *> TaskQueryDetailAttr(NRAttribute::TASK_QUERY_DETAIL_KEY, NRAttribute::BLOB_TYPE);


main()
{
   NRAttrVec attrs;
   /*
    * Demonstrate making some attributes.
    * All duplicitave infromation is in the factory.
    */

   attrs.push_back(LatitudeAttr.make(NRAttribute::IS, 45.1));
   attrs.push_back(HighPrecisionLatAttr.make(NRAttribute::IS, 46.10001));
   attrs.push_back(LongitudeAttr.make(NRAttribute::IS, 104.1));
   attrs.push_back(TaskNameAttr.make(NRAttribute::IS, "Harry Potter"));
   char *query_detail = "QD";
   attrs.push_back(TaskQueryDetailAttr.make(NRAttribute::IS, (void *)query_detail, strlen(query_detail) + 1));
   /* demonstrate multiple attrs with the same key */
   attrs.push_back(GroceriesAttr.make(NRAttribute::IS, "Lima beans"));
   attrs.push_back(GroceriesAttr.make(NRAttribute::IS, "Navy beans"));
   
   /*
    * Demonstrate extracting attributes.
    * With templates, attributes come out type safely
    * (users don't do casts).
    */
   NRSimpleAttribute<float>* lat = LatitudeAttr.find(&attrs);
   NRSimpleAttribute<float>* lon = LongitudeAttr.find(&attrs);
   NRSimpleAttribute<char *>* name = TaskNameAttr.find(&attrs);
   NRSimpleAttribute<void *>* qd = TaskQueryDetailAttr.find(&attrs);
   float lat_val = lat->getVal();
   cout << "lat: " << lat->getVal() <<
      " (hp-lat: " << HighPrecisionLatAttr.find(&attrs)->getVal() << ")" <<
      " lon: " << lon->getVal() <<
      " task-name: " << name->getVal() <<
      " query-detail: " << (char*)qd->getVal() <<
      "\n";

   /* Demonstrate extracting multiple attributes with same key. */
   NRAttrVec::iterator place = attrs.begin();
   for (;;) {
      NRSimpleAttribute<char*>* groc = GroceriesAttr.find_from(&attrs, place, &place);
      if (!groc)
	 break;
      cout << "  groc " << place << " " << groc->getVal() << endl;
      place++;
   };

   /* Demonstrate what happens when attributes don't exist. */
   NRSimpleAttribute<char *>* nongroc = NonGroceriesAttr.find(&attrs);
   assert(nongroc == NULL);

   printf("\nAdding one more item to the shopping list\n");

   /* Find an attribute like 'this' */
   NRAttribute *myattr = GroceriesAttr.make(NRAttribute::IS, "Chocolate Milk");
   attrs.push_back(myattr);
   place = attrs.begin();
   for (;;) {
      // This following line allows a later use of oldgroc->getVal()
      // NRSimpleAttribute<char *>* oldgroc = (NRSimpleAttribute<char *> *) myattr->find_matching_key_from(attrs, place, &place);

      NRAttribute *oldgroc = myattr->find_matching_key_from(&attrs, place, &place);
      if (!oldgroc)
	 break;
      cout << "  groc " << place << " " << (char *) oldgroc->getGenericVal() << endl;
      place++;
   };
   
   /*
    * walk the attributes in a generic way
    */
   // to be completed
}
