//
// UDPwired.cc   : UDP Wired Support for Diffusion
// Authors       : Chalermek Intanagonwiwat and Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: UDPwired.cc,v 1.3 2001/06/29 21:48:14 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include <stdio.h>
#include <stdlib.h>

#include "UDPwired.hh"

UDPWired::UDPWired(char *config_file) : DiffusionIO()
{
  Topology_Entry *entry;
  FILE *cfile;
  int flag, quality, port;
  struct socket_info *skt;
  char hname[50];
  struct hostent *hent = NULL;

  srand(time(NULL));

  // Try to open the config file
  cfile = fopen(config_file, "r");

  if (cfile == NULL){
    fprintf(stderr, "Can't find configuration file !\n");
    exit(-1);
  }

  // Parse the config file
  num_out_descriptors = 1;
  fprintf(stderr, "Parsing config file - %s \n", config_file);
  flag = fscanf(cfile, "%s - %d - %d\n", hname, &port, &quality);
  while (flag != EOF){
    if (flag == 0){
      fprintf(stderr, "Error in the config file !\n");
      exit(-1);
    }

    if (flag == 1){
      // Link quality and port not specified
      port = DEFAULT_DIFFUSION_PORT;
      quality = RELIABLE_LINK;
    }
    else{
      if (flag == 2){
	// We have either port or quality. Let's figure it out
	if (port > 0 && port <= RELIABLE_LINK){
	  // Ok, we have the link's quality
	  quality = port;
	  port = DEFAULT_DIFFUSION_PORT;
	}
	else{
	  if (port < 1024 || port > 65535){
	    fprintf(stderr, "Error parsing config file: Syntax error !\n");
	    exit(-1);
	  }
	  quality = RELIABLE_LINK;
	}
      }
      else{
	// We have port and quality
	if (quality < 0 || quality > RELIABLE_LINK){
	  fprintf(stderr, "Error parsing config file: Link quality outside range (0-100) !\n");
	  exit(-1);
	}
	if (port < 1024 || port > 65535){
	  fprintf(stderr, "Error parsing config file: Port outside range (1024-65535) !\n");
	  exit(-1);
	}
      }
    }

    fprintf(stderr, "Neighbor: %d - %s - Port: %d -  Link Quality: %d\n", num_out_descriptors, hname, port, quality);
    hent = gethostbyname(hname);
    if (hent == NULL){
      fprintf(stderr, "Cannot resolve host name !\n");
      exit(-1);
    }

    entry = new Topology_Entry;
    entry->fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (entry->fd < 0){
      fprintf(stderr, "Cannot create socket !\n");
      exit(-1);
    }

    bzero((char *) &(entry->skt_addr_inet), sizeof(entry->skt_addr_inet));

    // Configure socket
    entry->skt_addr_inet.sin_family = AF_INET;
    entry->skt_addr_inet.sin_port = htons((u_int16_t) port);
    bcopy(hent->h_addr, (char *)&(entry->skt_addr_inet.sin_addr),
	  hent->h_length);

    entry->quality = quality;

    neighbors_list.push_back(entry);
    num_out_descriptors++;

    flag = fscanf(cfile, "%s - %d - %d\n", hname, &port, &quality);
   }

  fclose(cfile);

  num_out_descriptors--;
  fprintf(stderr, "\nTotal Neighbors = %d\n", num_out_descriptors);
}

void UDPWired::SendPacket(Packet pkt, int len, int dst)
{
  TopologyList::iterator itr;
  Topology_Entry *entry;
  int prob_loss;

  if (len > MAX_UDP_MESSAGE_SIZE){
    fprintf(stderr, "Warning: Packet larger than maximum UDP datagram size !\n");
    exit(-1);
  }

  for (itr = neighbors_list.begin(); itr != neighbors_list.end(); ++itr){
    entry = *itr;
    if (entry->quality == RELIABLE_LINK){
      sendto(entry->fd, pkt, len, 0,
	     (struct sockaddr *)&entry->skt_addr_inet,
	     sizeof(entry->skt_addr_inet));
    }
    else{
      prob_loss = (int) (RELIABLE_LINK * (rand() * 1.0 / RAND_MAX));
      if (prob_loss <= entry->quality)
	sendto(entry->fd, pkt, len, 0,
	       (struct sockaddr *)&entry->skt_addr_inet,
	       sizeof(entry->skt_addr_inet));
      else
	fprintf(stderr, "Packet dropped !\n");
    }
  }
}
