//
// WINSng2.hh    : WINSng2 Radio Support Include File
// Authors       : Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: WINSng2.hh,v 1.3 2001/07/12 02:01:09 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#ifndef winsng2_hh
#define winsng2_hh

#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 

#include "iodev.hh"
#include "rf.h"

#define RADIO_ID_LEN            3
#define DIFFUSION_PACKET_TYPE  60
#define WINS_BROADCAST        255

// Raw device files
// #define RADIO_DEVICE_FILE_1 "/dev/rf1"
// #define RADIO_DEVICE_FILE_2 "/dev/rf2"

// Streamd device files
#define RADIO_DEVICE_FILE_1 "/dev/rf1c"
#define RADIO_DEVICE_FILE_2 "/dev/rf2c"

struct wins_header {
  uint16 length;
  uint8 source[3];
  uint8 destination[3];
  uint8 type;
};

class RFIdCacheEntry {
public:
  int32_t diffusionId;
  uint8 radioId[RADIO_ID_LEN];
  int   radiofd;
};

typedef list<RFIdCacheEntry *> RFIdCache;

class WINSNG2 : public DiffusionIO {
public:
  WINSNG2();
  Packet RecvPacket(int fd);
  void SendPacket(Packet pkt, int len, int dst);

protected:
  RFIdCache RfCache;
  void AddCache(int fd, int32_t diffId, uint8 *srcId);
  int FindInCache(int32_t diffId, uint8 *srcId);
  char * AssemblePacket(uint8 *srcId, int len, Packet pkt);
};

#endif // winsng2_hh
